.g-oidc-login-container
  margin-top: 20px
  
  .panel
    border-radius: 4px
    
    .panel-body
      text-align: center
      padding: 20px
      
      h3
        margin-bottom: 15px
        color: #333
      
      .g-oidc-login
        font-size: 16px
        padding: 10px 20px
        
        i
          margin-right: 8px
      
      .g-oidc-error
        margin-top: 15px
        animation: fadeIn 0.3s
