.g-oidc-account-view
  display: flex
  justify-content: center
  align-items: center
  min-height: 400px
  padding: 20px

  .g-oidc-account-message
    width: 100%
    max-width: 600px
    text-align: center

    .alert
      margin: 0
      border-radius: 4px
      padding: 20px

      h4
        margin-top: 0
        margin-bottom: 15px
        color: #004085

        i
          margin-right: 8px

      p
        margin-bottom: 10px
        line-height: 1.6

        &:last-child
          margin-bottom: 0

        &:last-child
          margin-bottom: 0
